function varargout = M3DET_v7(varargin)
%Ersan Turkoglu, October 27 2008, version-7, 
%Problems fixed: 1-When winglink model loaded depth layers were not plotted correctly so user could not edit layers but model saved correctly.
%Improvements : 1-if you have ocean in the model program can understand and
%fix these layers resistivity during the inversion and writes modelfilename_cmi file cmi=control model index,
%CMI file has to be defined in the startup file to fix it or use default just to have ocean in the apriori and starting model. 
%2-If you load a model with more than 9 distinct resistivity values program
%will understand this and simplifies your model down to 9 values that you
%can define. Remember to enter the first value less than one if model has
%ocean.
%To be able to use this feature export your final model from S3DET_v5 then
%load to M3DET_v7 (load winglink model)
%--------------
%Ersan Turkoglu, August 12 2008, version-6, impedance data is rotated opposite direction (respect to winglink) of the mesh rotation
%note that X is assumed to be north in the inversion which is not the case
%for winglink 3D inversion
%Ersan Turkoglu, August 08 2008, version-5, undo and redo works better
%Ersan Turkoglu, July 29 2008, version-4, a bug with depths when model
%generated in this program is fixed.
%Ersan Turkoglu, July 23 2008, version-3, can load previously made wsinv3dmt
%models, better structure.
% M3DET_V7 M-file for M3DET_v7.fig
%      M3DET_V7, by itself, creates a new M3DET_V7 or raises the existing
%      singleton*.
%
%      H = M3DET_V7 returns the handle to a new M3DET_V7 or the handle to
%      the existing singleton*.
%
%      M3DET_V7('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in M3DET_V7.M with the given inputs
%      arguments.
%
%      M3DET_V7('Property','Value',...) creates a new M3DET_V7 or raises
%      the
%      existing singleton*.  Starting from the left, property value pairs
%      are
%      applied to the GUI before M3DET_v4_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property
%      application
%      stop.  All inputs are passed to M3DET_v7_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help M3DET_v7

% Last Modified by GUIDE v2.5 31-Oct-2008 00:23:26

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @M3DET_v7_OpeningFcn, ...
    'gui_OutputFcn',  @M3DET_v7_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end

% End initialization code - DO NOT EDIT


% --- Executes just before M3DET_v7 is made visible.
function M3DET_v7_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to M3DET_v7 (see VARARGIN)

% Choose default command line output for M3DET_v7

handles.output = hObject;
global undo
delete undo*
undo=0;
set(hObject,'Toolbar','figure');  % Display the standard toolbar
disp('THIS PROGRAM PREPARES 3D MESH GRID AND DATA FILES FOR WSINV3DMT PROGRAM')
disp('Ersan Turkoglu, University of Alberta, March, 2006')
disp('This program prepares input files for WSINV3DMT')
disp('May not be used or distributed without permission')
disp('Users agree to report any moderate or major changes made to this program')
disp('contact for permission and technical support: ersan.turkoglu@gmail.com')

[x,y,z] = meshgrid(-2:.2:2, -2:.25:2, -2:.16:2);
v = x .* exp(-x.^2 - y.^2 - z.^2);
slice(x,y,z,v,[-1.2 .8 2],2,[-2 -.2])
axis equal; axis tight; clear x y z
xlb='Easting (km)'; ylb='Northing (km)'; zlb='Depth (km)';
xlabel(xlb);ylabel(ylb);zlabel(zlb)
axis off; % Turn off all axis labeling
set(handles.axes1,'HandleVisibility','OFF');

% Update handles structure
guidata(hObject, handles)
% UIWAIT makes M3DET_v7 wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = M3DET_v7_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;



% --- Executes on button press in newmodel.
function newmodel_Callback(hObject, eventdata, handles)
% hObject    handle to newmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
%handles.output = hObject

clear x y per nx ny nz Z XX YY x_orig y_orig lay matfile undo
delete undo*
global x y per nx ny nz Z XX YY x_orig y_orig lay matfile undo
lay=1;

[cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

set(handles.axes1,'HandleVisibility','ON');
axes(handles.axes1)

% load impedance array
[matfile,matpath] = uigetfile({'*.mat'},'pick mat file');
eval(['load ',matfile]);
per = T; logper = log10(per); nper = length(per); nsites = size(coord,1);
lat  = coord(:,2)'; long = coord(:,1)';
cent_lat  = (max(lat)+min(lat))/2;
cent_long = (max(long)+min(long))/2;
% Convert degrees to km N-S and E-W
x = 111.*cos(cent_lat*pi/180)*(long-cent_long);
y = 111.*(lat-cent_lat);
x_orig=x; y_orig=y;
plot(x,y,'vk','markerfacecolor','b'); axis equal; xlabel('Easting (km)'); ylabel('Northing (km)')
axis([min(x)-sttol max(x)+sttol min(y)-sttol max(y)+sttol]);
text(x,y,site,'fontsize',6);


data=guidata(hObject);
%zooms= zoom;; set(zooms,'Enable','on');
guidata(hObject, handles);

function [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles)
min_res =log10(str2double(get(handles.min_res,'string'))); % Get the min resistivity
max_res =log10(str2double(get(handles.max_res,'string'))); % Get the max resistivity
colim=[min_res-0.15 max_res+0.15]; % caxis values for color plots
%0.15 to make figures same as pmv* because WS did so...
cmap=jet(24);
cmap([23 21 19 17 15 13 11 9 7 5 3],:)=[];
cmap=flipud(cmap);
sttol=(str2double(get(handles.stat_tol,'string'))); % station distance tolerance


% --- Executes on button press in loadmodel.
function loadmodel_Callback(hObject, eventdata, handles)
% hObject    handle to loadmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
clear global
global x y per nx ny nz Z XX YY x_orig y_orig AA lay matfile mes_rot rot_cor_ang
lay=1;
[cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

set(handles.axes1,'HandleVisibility','ON');
axes(handles.axes1)

%Load model
[meshf,matpath] = uigetfile({'*.out'},'pick winglink *.out file');
fid=fopen(meshf,'r');
[NN]=fscanf(fid,'%d %d %d %d');
nx=NN(1);ny=NN(2);nz=NN(3);na=NN(4);
%winglink uses mathematical coordinates WSINV3DMT uses geographical
%coordinates
bx=fscanf(fid,'%s',1); XX=fscanf(fid,'%f',nx);
YY=fscanf(fid,'%f',ny);
Z=round(fscanf(fid,'%f',nz));
slj=[];
for uu=1:nz
    nul=fscanf(fid,'%f',1); %We don't need this
    sli=fscanf(fid,'%f',nx*ny);
    slj=abs([slj; sli]);
end
for ofbe=1:5
    ofb=fgetl(fid);
end
mes_rot=-fscanf(fid,'%f',1); %if this is nonzero station coordinate frame will be rotated.
res=unique(slj);

if length(res)>9
    warndlg('You cannot have more than 9 resistivities, program will simplify your model');

    resis9=round(logspace(0,3,9));
    prompt={'Enter the resistivity values to index the model - increasing resistivity'};
    name='Maximum 9 values-increasing resistivity order';
    numlines=1;
    defaultanswer={num2str(resis9)};
    answer=inputdlg(prompt,name,numlines,defaultanswer);
    resis9=str2num(char(answer));
    resdif=diff(resis9); resdif=[resdif max(resdif)*100];
    %slj(find(slj<min(resis9)))=resis9(2);

    for k=1:8
        slj(find(slj>resis9(k)+resdif(k)/2 & slj<=resis9(k+1)+resdif(k+1)/2))=resis9(k+1);
    end
    slj(find(slj>resis9(1) & slj<resis9(2)))=resis9(2);
    if length(find(slj<resis9(1)))>0
        warndlg('Model has resistivities lower than your lowest resistivity value. To avoid complications these values are replaced by your second resistivity value.');
        slj(find(slj<resis9(1)))=resis9(2);
    end
        
end
res=unique(slj);

AA=reshape(slj,nx,ny,nz);
AAA(:,1,:)=AA(:,ny,:);
AAA(:,ny,:)=AA(:,1,:);
for ty=2:ny-1
    AAA(:,ty,:)=AA(:,ny-ty+1,:);
end
AA=AAA;

%Program will only work for nine distinct resistivity values in model file

nnr= length(res);
fclose(fid);
%
for opi=1:length(res)
    slj(find(slj==res(opi)))=opi;
end
% load Data
[matfile,matpath] = uigetfile({'*.mat'},'pick mat file');
eval(['load ',matfile]);
per = T; logper = log10(per); nper = length(per); nsites = size(coord,1);
lat  = coord(:,2)'; long = coord(:,1)';
cent_lat  = (max(lat)+min(lat))/2;
cent_long = (max(long)+min(long))/2;
% Convert degrees to km N-S and E-W
x = 111.*cos(cent_lat*pi/180)*(long-cent_long);
y = 111.*(lat-cent_lat);
x_orig=x; y_orig=y;
%rotate stations if mesh_rot is nonzero
if mes_rot~=0
    rot_cor_ang=mes_rot;
    c = cos(rot_cor_ang*pi/180);    s = sin(rot_cor_ang*pi/180);
    rotmat=[c -s;s c];
    rotated=rotmat*[x_orig' y_orig']';
    x=rotated(1,:);y=rotated(2,:);
    axis equal; xlabel('Easting (km)'); ylabel('Northing (km)')
    plot(x_orig,y_orig,'vk','markerfacecolor','g'); hold on;
    plot(x,y,'vk','markerfacecolor','b');
    legend('original locations','rotated locations')
    axis([min([x x_orig])-sttol max([x x_orig])+sttol min([y y_orig])-sttol max([y y_orig])+sttol]);
    view_model_Callback(hObject, eventdata, handles)
    text(x,y,site,'fontsize',6);
else
    plot(x,y,'vk','markerfacecolor','b');
    axis([min([x x_orig])-sttol max([x x_orig])+sttol min([y y_orig])-sttol max([y y_orig])+sttol]);
    view_model_Callback(hObject, eventdata, handles)
    text(x,y,site,'fontsize',6);
end
%caxis(colim); colormap(cmap);
%hcb=colorbar('ver','position',[0.655 0.2138 0.02309 0.3977]);
%set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
%    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
%set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

data=guidata(hObject);
guidata(hObject, handles);


% --- Executes on button press in savemodel.
function savemodel_Callback(hObject, eventdata, handles)
% hObject    handle to savemodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global x y per nx ny nz Z XX YY AA res
%If you haven't make the model you can't save it
sto=0; sto1=0;
if min(XX)<0  %This will work if you generated the model instead of loading from winglink
    AAA=AA;
    xx=diff(XX)*1000;
    yy=diff(YY)*1000;
    AAA(nx,:,:)=[];
    AAA(:,ny,:)=[];
    AAA(:,:,nz)=[];
    nxx=nx-1;
    nyy=ny-1;
    nzz=nz-1;
    if Z(3)/Z(2)>2
        ZZ=diff(Z)*1000;
    else
        ZZ=Z*1000;
        ZZ(1)=[];
    end
else %This will work if you loaded a winglink model
    AAA=AA;
    xx=XX;
    yy=YY;
    nxx=nx;
    nyy=ny;
    nzz=nz;
    ZZ=Z;
end

if length(XX)<5
    warndlg('Make a model before you save it!')
    return
else
    ocean =get(handles.checkbox5,'value'); % is there ocean in the model
    fix_ocean =get(handles.checkbox6,'value'); % fix ocean resistivity?
    ocean_res =str2double(get(handles.edit35,'string')); % ocean resistivity
    
    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1)
    %----------------Model Verification---------------------
    if rem(nxx,2)==1 & rem(nyy,2)==1
        warndlg('model has odd number of columns and rows, model not saved. make them even')
        sto1=1;
    elseif rem(nxx,2)==1
        warndlg('model has odd number of columns, model not saved. make it even')
        sto1=1;
    elseif rem(nyy,2)==1
        warndlg('model has odd number of rows, model not saved. make it even')
        sto1=1;
    end
    for i=1:nxx-1
        stv=find(XX(i)<x & x<XX(i+1));%  stations in the first vertical mesh cell
        if isempty(stv)==0
            for j=1:ny-1
                sty=find(YY(j)<y(stv) & y(stv)<YY(j+1)); % stations fall in the same cell
                if length(sty)>1
                    hold on;plot(x(stv(sty)),y(stv(sty)),'vr','markerfacecolor','r','markersize',12);hold off;
                    sto=1;
                end
            end
        end
    end
    if sto==1
        warndlg('multiple stations in one cell, model not saved')
    end
    ress=unique(AAA);

    if length(ress)>9
        warndlg('You cannot have more than 9 resistivities, model not saved');
        sto1=1;
    end
    %----------------saves initial model file---------------
    if sto==0 && sto1==0
        nnr=length(ress);
        for opi=1:length(ress)
            AAA(find(AAA==ress(opi)))=opi;
        end
        def = {'data_name'};
        prompt = {'enter model file name no extensions'};
        titles  = 'Model file name';
        modf = char(inputdlg(prompt,titles,1,def)); %model file name

        while exist(modf)>0
            newmod_name = questdlg('This file exist! Do you want to overwrite the file?', ...
                'The model file exist', ...
                'Yes', 'No','dummy');
            switch newmod_name,
                case 'Yes',
                    eval(['delete ',modf]);
                    break
                case 'No',
                    def = {'data_name'};
                    prompt = {'enter a new model file name (no extensions)'};
                    titles  = 'Model file name';
                    modf = char(inputdlg(prompt,titles,1,def)); %model file name
            end % switch
        end

        %Compare each layer with the next to find depth interfaces (dint)
        aa=1;
        slj=reshape(AAA,nxx*nyy*nzz,1);
        for uu=1:nzz-1
            if slj(aa:uu*nxx*nyy)==slj(uu*nxx*nyy+1:(uu+1)*nxx*nyy)
            else
                dint(uu)=uu+1;
            end
            aa=uu*nxx*nyy+1;
        end
        if exist('dint')==1
            dint=dint(find(dint~=0)); %layer numbers
            dint=[1 dint nzz+1];
        else
            dint=[1 5 nzz+1];
            nnr=2;
            ress=[ress ress];
        end
        AAA=reshape(slj,nxx,nyy,nzz);
        if ocean==1
            ress(find(ress)<1)=ocean_res;
            if fix_ocean==1
                CMI=AAA; CMI(find(CMI~=1))=0; %everything is zero except ocean CMI=control model index
                %generate control model index ---used to fix some part of the
                %model such as ocean 0 means free 1 means freezed/fixed.
                fid=fopen([modf,'_cmi'],'w');
                %fprintf(fid,'%s\n','# CONTROL MODEL INDEX');
                %this is strange here now--- X and Y will change here to match the
                %program requirements. This is very strange but IS correct.
                fprintf(fid,'%d %d %d\n',nyy,nxx,nzz);
                %For layered model with up to 9 distinct resistivities
                for kj=1:length(dint)-1
                    fprintf(fid,'%d %d\n',dint(kj),dint(kj+1)-1);    %layers
                    for i=nyy:-1:1
                        fprintf(fid,'%d ',CMI(:,i,dint(kj)));
                        if kj==length(dint)-1 && i==1
                        else
                            fprintf(fid,'\n');
                        end
                    end
                end
                fclose(fid);
            end
        end
        %         for pl=1:nz
        %             sld(:,:,pl)=slh(:,:,pl)';
        %         end
        %generate initial model
        if find(ress<1)>0 & fix_ocean==0; warndlg('Your model has resistivity less than 1, you probably have ocean in your model, you may fix ocean during the inversion. Use "Ocean Model" Module');end
        fid=fopen(modf,'w');
        fprintf(fid,'%s\n','# INITIAL MODEL');
        %this is strange here now--- X and Y will change here to match the
        %program requirements. This is very strange but IS correct.
        fprintf(fid,'%d %d %d %d\n',nyy,nxx,nzz,nnr);
        fprintf(fid,'%.0f ',yy);
        fprintf(fid,'\n');
        fprintf(fid,'%.0f ',xx);
        fprintf(fid,'\n');
        fprintf(fid,'%.0f ',ZZ);
        fprintf(fid,'\n');
        fprintf(fid,'%.2f ',ress);
        fprintf(fid,' ');
        fprintf(fid,'\n');
        %For layered model with up to 9 distinct resistivities
        for kj=1:length(dint)-1
            fprintf(fid,'%d %d\n',dint(kj),dint(kj+1)-1);    %layers
            for i=nyy:-1:1
                fprintf(fid,'%d ',AAA(:,i,dint(kj)));
                if kj==length(dint)-1 && i==1
                else
                    fprintf(fid,'\n');
                end
            end
        end
        fclose(fid);
      
        data=guidata(hObject);
        guidata(hObject, handles);
    end
end



% --- Executes on button press in exit.
function exit_Callback(hObject, eventdata, handles)
% hObject    handle to exit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
%exit the program completely
close all; clear all
delete undo*


function xspacing_Callback(hObject, eventdata, handles)
% hObject    handle to xspacing (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of xspacing as text
%        str2double(get(hObject,'String')) returns contents of xspacing as a double


% --- Executes during object creation, after setting all properties.
function xspacing_CreateFcn(hObject, eventdata, handles)
% hObject    handle to xspacing (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function yspacing_Callback(hObject, eventdata, handles)
% hObject    handle to yspacing (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of yspacing as text
%        str2double(get(hObject,'String')) returns contents of yspacing as a double


% --- Executes during object creation, after setting all properties.
function yspacing_CreateFcn(hObject, eventdata, handles)
% hObject    handle to yspacing (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function firstthick_Callback(hObject, eventdata, handles)
% hObject    handle to firstthick (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of firstthick as text
%        str2double(get(hObject,'String')) returns contents of firstthick as a double


% --- Executes during object creation, after setting all properties.
function firstthick_CreateFcn(hObject, eventdata, handles)
% hObject    handle to firstthick (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function xoffmodel_Callback(hObject, eventdata, handles)
% hObject    handle to xoffmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of xoffmodel as text
%        str2double(get(hObject,'String')) returns contents of xoffmodel as a double


% --- Executes during object creation, after setting all properties.
function xoffmodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to xoffmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function yoffmodel_Callback(hObject, eventdata, handles)
% hObject    handle to yoffmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of yoffmodel as text
%        str2double(get(hObject,'String')) returns contents of yoffmodel as a double


% --- Executes during object creation, after setting all properties.
function yoffmodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to yoffmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function zoffmodel_Callback(hObject, eventdata, handles)
% hObject    handle to zoffmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of zoffmodel as text
%        str2double(get(hObject,'String')) returns contents of zoffmodel as a double


% --- Executes during object creation, after setting all properties.
function zoffmodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to zoffmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in generatemodel.
function generatemodel_Callback(hObject, eventdata, handles)
% hObject    handle to generatemodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo
lay=1;
if length(x)<1
    warndlg('You have to load a *.mat data file before you can generate the mode. Click "New Model" first.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);

    %default resistivity of the model
    res =str2double(get(handles.resistivity,'string')); % Get the resistivity
    xspacing = str2double(get(handles.xspacing,'string')); % Get the xspacing
    moutcX = str2double(get(handles.moutcX,'string')); % Number of mesh out of the core in X
    yspacing = str2double(get(handles.yspacing,'string')); % Get the yspacing
    moutcY = str2double(get(handles.moutcY,'string')); % Number of mesh out of the core in Y
    firstthick = str2double(get(handles.firstthick,'string')); % Get the first thickness
    nz = str2double(get(handles.nl,'string')); % Get the number of layers
    xcoef = str2double(get(handles.xoffmodel,'string')); % Get the coefficient for off model in X direction
    ycoef = str2double(get(handles.yoffmodel,'string')); % Get the coefficient for off model in Y direction
    zcoef = str2double(get(handles.zoffmodel,'string')); % Get the coefficient for model in Z direction

    lomax=(max(x)-min(x))/2 ;
    nx=(ceil(lomax/xspacing)+4)*xspacing;
    X1=linspace(0,nx,(ceil(lomax/xspacing)+5))-xspacing; X1(1:2)=[]; X2=-X1;
    X=[X2,X1,0];
    lamax=(max(y)-min(y))/2 ;
    ny=(ceil(lamax/yspacing)+5)*yspacing;
    Y1=linspace(0,ny,(ceil(lamax/yspacing)+6))-yspacing;
    Y1(1:2)=[]; Y2=-Y1;
    Y=[Y2,Y1,0];
    XX=sort(X); YY=sort(Y);
    xx(1)=XX(1)+(XX(1)-XX(2))*xcoef;
    xx(2)=xx(1)+(xx(1)-XX(1))*xcoef;
    for i=3:moutcX
        xx(i)=xx(i-1)+(xx(i-1)-xx(i-2))*xcoef;
    end
    yy(1)=YY(1)+(YY(1)-YY(2))*ycoef;
    yy(2)=yy(1)+(yy(1)-YY(1))*ycoef;
    for i=3:moutcY
        yy(i)=yy(i-1)+(yy(i-1)-yy(i-2))*ycoef;
    end
    %thickness
    z(1)=(firstthick);
    for i=2:nz
        z(i)=z(i-1)*zcoef;
    end
    z=cumsum(z);z=sort([z,0]);
    Z=round(z)/1000; XX=sort([XX,xx,-xx]); YY=sort([YY,yy,-yy])'; %x, y and z defined as vectors

    nx=length(XX); ny=length(YY); nz=length(Z);
    AA=ones(nx,ny,nz)*res; %Default model
    aa=reshape(AA(:,:,1),nx,ny);
    pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
    axis equal; axis tight
    title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in addrow.
function addrow_Callback(hObject, eventdata, handles)
% hObject    handle to addrow (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);
    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);


    while 1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom; set(zooms,'Enable','on');
            break
        end
        zb=round(zb*1000)/1000;minza=min(abs(abs(YY)-abs(zb)));
        if minza<1   %minimum mesh spacing is 1km
            button = questdlg('rows are too close! Do you want to add anyways?','Are you sure','Yes','No','default');
            if length(button)==3
                YY=sort([YY;zb]); lk=find(YY==zb);
                nx=length(XX); ny=length(YY);
                ab(:,1:lk,:)=AA(:,1:lk,:); ab(:,lk+1,:)=AA(:,lk,:); ab(:,lk+2:ny,:)=AA(:,lk+1:ny-1,:); AA=ab;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
            end
        else
            YY=sort([YY;zb]); lk=find(YY==zb);
            nx=length(XX); ny=length(YY);
            ab(:,1:lk,:)=AA(:,1:lk,:); ab(:,lk+1,:)=AA(:,lk,:); ab(:,lk+2:ny,:)=AA(:,lk+1:ny-1,:); AA=ab;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
        end
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
    end
    %update handles
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    guidata(hObject, handles)
end

% --- Executes on button press in deleterow.
function deleterow_Callback(hObject, eventdata, handles)
% hObject    handle to deleterow (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);
    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');

    while -1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom; set(zooms,'Enable','on');
            break
        end
        xbu=abs(YY-zb); xbu=find(min(xbu)==xbu);
        YY(xbu)=[];
        nx=length(XX); ny=length(YY);
        AA(:,xbu,:)=[];
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
        set(handles.axes1,'XLim',newXLim)
        set(handles.axes1,'YLim',newYLim)
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
    end
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    %update handles
    guidata(hObject, handles)
end

% --- Executes on button press in addcolumn.
function addcolumn_Callback(hObject, eventdata, handles)
% hObject    handle to addcolumn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);
    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');

    while 1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom; set(zooms,'Enable','on');
            break
        end
        za=round(za*1000)/1000;minza=min(abs(abs(XX)-abs(za)));
        if minza<1   %minimum mesh spacing is 1km
            button = questdlg('Columns are too close! Do you want to add anyways?','Are you sure','Yes','No','default');
            if length(button)==3
                XX=sort([XX,za]);  lk=find(XX==za);
                nx=length(XX); ny=length(YY);
                ab(1:lk,:,:)=AA(1:lk,:,:); ab(lk+1,:,:)=AA(lk,:,:); ab(lk+2:nx,:,:)=AA(lk+1:nx-1,:,:); AA=ab;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight
            end
        else
            [yut mam]=size(XX);
            if yut>1
                XX=sort([XX;za]);
            else
                XX=sort([XX,za]);
            end
            lk=find(XX==za);
            nx=length(XX); ny=length(YY);
            ab(1:lk,:,:)=AA(1:lk,:,:); ab(lk+1,:,:)=AA(lk,:,:); ab(lk+2:nx,:,:)=AA(lk+1:nx-1,:,:); AA=ab;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight
        end
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
    end
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    %update handles
    guidata(hObject, handles)
end

% --- Executes on button press in deletecolumn.
function deletecolumn_Callback(hObject, eventdata, handles)
% hObject    handle to deletecolumn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay undo
if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);
    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');

    while 1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom; set(zooms,'Enable','on');
            break
        end
        size (XX)
        xbu=abs(XX-za); xbu=find(min(xbu)==xbu);
        XX(xbu)=[];
        AA(xbu,:,:)=[];
        nx=length(XX); ny=length(YY);
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
        set(handles.axes1,'XLim',newXLim)
        set(handles.axes1,'YLim',newYLim)
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
    end
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end


% --- Executes on button press in movecolumn.
function movecolumn_Callback(hObject, eventdata, handles)
% hObject    handle to movecolumn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA
if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    view_model_Callback(hObject, eventdata, handles)
end

% --- Executes on button press in moverow.
function moverow_Callback(hObject, eventdata, handles)
% hObject    handle to moverow (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    view_model_Callback(hObject, eventdata, handles)

end

% --- Executes on button press in addlayer.
function addlayer_Callback(hObject, eventdata, handles)
% hObject    handle to addlayer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);
    aa=reshape(AA(:,lay,:),nx,nz);
    
    if Z(3)/Z(2)<1.8
        Z=cumsum(Z);
    end
    pcolor(XX,Z,aa'); %hold on; plot(x,y,'vk'); hold off
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

    xlabel('Y (km)'); ylabel('Depth (km)');
    set(gca,'YDir','reverse');
    title('zoom in first then click "Continue" to add layers')

    data=guidata(gca);
    set(data.con_tinue,'visible','on')
    uiwait(gcf); % This allows zooming

    newXLim=get(handles.axes1,'XLim');
    newYLim=get(handles.axes1,'YLim');
    while 1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom;; set(zooms,'Enable','on');
            break
        end
        zb=round(zb*1000)/1000;minza=min(abs(abs(YY)-abs(zb)));
        if minza<.5   %minimum layer spacing is 80m
            button = questdlg('layers are too close! Do you want to add anyways?','Are you sure','Yes','No','default');
            if length(button)==3
                Z=sort([Z,zb]); kl=find(Z==zb);
                nx=length(XX); ny=length(YY); nz=length(Z);
                ab(:,:,1:kl)=AA(:,:,1:kl); ab(:,:,kl+1)=AA(:,:,kl); ab(:,:,kl+2:nz)=AA(:,:,kl+1:nz-1); AA=ab;
                aa=reshape(AA(:,lay,:),nx,nz);
                pcolor(XX,Z,aa'); %hold on; plot(x,y,'vk'); hold off
                xlabel('Y (km)'); ylabel('Depth (km)');
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                set(gca,'YDir','reverse');
            end
        else
            okp=size(Z);
            if okp(1)>1
                Z=Z';
            end
            Z=sort([Z,zb]); kl=find(Z==zb);
            nx=length(XX); ny=length(YY); nz=length(Z);
            ab(:,:,1:kl)=AA(:,:,1:kl); ab(:,:,kl+1)=AA(:,:,kl); ab(:,:,kl+2:nz)=AA(:,:,kl+1:nz-1); AA=ab;
            aa=reshape(AA(:,lay,:),nx,nz);
            pcolor(XX,Z,aa'); %hold on; plot(x,y,'vk'); hold off
            xlabel('Y (km)'); ylabel('Depth (km)');
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
            set(gca,'YDir','reverse');
        end
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

    end
    %update handles
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in deletelayer.
function deletelayer_Callback(hObject, eventdata, handles)
% hObject    handle to deletelayer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    if Z(3)/Z(2)<1.8
        Z=cumsum(Z);
    end

    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);

    aa=reshape(AA(:,lay,:),nx,nz);
    pcolor(XX,Z,aa'); %hold on; plot(x,y,'vk'); hold off
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

    xlabel('Y (km)'); ylabel('Depth (km)');
    set(gca,'YDir','reverse');
    title('zoom in first then click "Continue" to delete layers')

    data=guidata(gca);
    set(data.con_tinue,'visible','on')
    uiwait(gcf); %This allows zooming

    newXLim=get(handles.axes1,'XLim');
    newYLim=get(handles.axes1,'YLim');

    while 1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom;; set(zooms,'Enable','on');
            break
        end
        xbu=abs(Z-zb); xbu=find(min(xbu)==xbu);
        Z(xbu)=[];
        nx=length(XX); ny=length(YY); nz=length(Z);
        AA(:,:,xbu)=[];
        aa=reshape(AA(:,lay,:),nx,nz);
        pcolor(XX,Z,aa');
        xlabel('Y (km)'); ylabel('Depth (km)');
        set(handles.axes1,'XLim',newXLim)
        set(handles.axes1,'YLim',newYLim)
        set(gca,'YDir','reverse');
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

    end
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    guidata(hObject, handles)
end

% --- Executes on button press in movelayer.
function movelayer_Callback(hObject, eventdata, handles)
% hObject    handle to movelayer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY AA lay


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    view_model_Callback(hObject, eventdata, handles)

end

function moutcX_Callback(hObject, eventdata, handles)
% hObject    handle to moutcX (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of moutcX as text
%        str2double(get(hObject,'String')) returns contents of moutcX as a double


% --- Executes during object creation, after setting all properties.
function moutcX_CreateFcn(hObject, eventdata, handles)
% hObject    handle to moutcX (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function moutcY_Callback(hObject, eventdata, handles)
% hObject    handle to moutcY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of moutcY as text
%        str2double(get(hObject,'String')) returns contents of moutcY as a double


% --- Executes during object creation, after setting all properties.
function moutcY_CreateFcn(hObject, eventdata, handles)
% hObject    handle to moutcY (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function nl_Callback(hObject, eventdata, handles)
% hObject    handle to nl (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of nl as text
%        str2double(get(hObject,'String')) returns contents of nl as a double


% --- Executes during object creation, after setting all properties.
function nl_CreateFcn(hObject, eventdata, handles)
% hObject    handle to nl (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end




% --- Executes on button press in destroyrow.
function destroyrow_Callback(hObject, eventdata, handles)
% hObject    handle to destroyrow (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of destroyrow

global x y per nx ny nz Z XX YY AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);
    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');

    while -1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom;; set(zooms,'Enable','on');
            break
        end
        xbu=abs(YY-zb); xbu1=find(min(xbu)==xbu); xbu=xbu1; xbu2=YY(xbu);
        YY(xbu)=[];
        YY(xbu:ny-1)=YY(xbu:ny-1)-xbu2+YY(xbu-1);
        AA(:,xbu,:)=[];
        nx=length(XX); ny=length(YY);
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        set(handles.axes1,'XLim',newXLim)
        set(handles.axes1,'YLim',newYLim)
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
    end
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end


% --- Executes on button press in destroycolumn.
function destroycolumn_Callback(hObject, eventdata, handles)
% hObject    handle to destroycolumn (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% Hint: get(hObject,'Value') returns toggle state of destroycolumn
global x y per nx ny nz Z XX YY AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    while -1
        [za,zb,zc]=ginput(1);
        if zc==3
            zooms= zoom;; set(zooms,'Enable','on');
            break
        end
        xbu=abs(XX-za); xbu1=find(min(xbu)==xbu); xbu=xbu1; xbu2=XX(xbu);
        XX(xbu)=[];
        XX(xbu:nx-1)=XX(xbu:nx-1)-xbu2+XX(xbu-1);
        nx=length(XX); ny=length(YY);
        AA(xbu,:,:)=[];
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        set(handles.axes1,'XLim',newXLim)
        set(handles.axes1,'YLim',newYLim)
        title(['number of X=',num2str(nx-1),', number of Y=',num2str(ny-1),', number of Z=',num2str(nz-1)]);
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

    end
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    %update handles
    guidata(hObject, handles)
end


% --- Executes on button press in station_sel.
function station_sel_Callback(hObject, eventdata, handles)
% Station selection
% hObject    handle to station_sel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY x_orig y_orig AA matfile

eval(['load ',matfile]);
if length(x)<1
    warndlg('You have to load a *.mat data file before you can select stations. Click "New Model" first.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1);
    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    stps=[];
    while 1
        [exs yes click]=ginput(1);
        if click==3;
            x(stps)=[];y(stps)=[];  %stations are eliminated.
            EEE(:,:,stps)=[]; EEE_HZ(:,:,stps)=[]; ZZZ(:,:,stps)=[]; ZZZ_HZ(:,:,stps)=[]; coord(stps,:)=[]; site(stps)=[];
            matfile_or=matfile; nokta=find(matfile=='.'); matfile(nokta:nokta+8)='_less.mat';
            eval(['save ',matfile,' T EEE EEE_HZ ZZZ ZZZ_HZ coord site']);
            hold off;
            plot(x,y,'vr','markerfacecolor','r')
            title('only these stations will be used')
            axis equal
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
            x_orig=x; y_orig=y;
            warndlg(['original data file ',matfile_or,' modified and saved as ',matfile])
            break
        end
        stp1=find(y>yes-sttol/2 & y<yes+sttol/2 & x>exs-sttol/2 & x<exs+sttol/2);
        stp1=min(stp1);
        if stp1>0
            dec=find(stps==stp1);
            if dec>0
                stps(dec)=[];
            else
                stps=[stps,stp1];
            end
            hold on; plot(x,y,'vk','markerfacecolor','b'); plot(x(stps),y(stps),'vk','markerfacecolor','r')
        end
        axis equal
        set(handles.axes1,'XLim',newXLim)
        set(handles.axes1,'YLim',newYLim)
    end
    %update handles
    guidata(hObject, handles)
end


% --- Executes on button press in view_model.
function view_model_Callback(hObject, eventdata, handles)
% hObject    handle to view_model (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay
if length(XX)<5
    warndlg('Generate a model then view it.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);
    if min(XX)>0 %this means you have just loaded data from winglink or loading previous wsinw3dmt model!

        cvx=cumsum(flipud(XX(1:nx/2)));
        cvxx=cumsum((XX(nx/2+1:nx)));
        XX=([flipud(-cvx)',0,cvxx']/1000)';

        cvy=cumsum(flipud(YY(1:ny/2)));
        cvyy=cumsum((YY(ny/2+1:ny)));
        YY=([flipud(-cvy)',0,cvyy']/1000)';
        %make zero point in the middle
        %These lines are not working yet disregard
%         sp=1;
%         yy=YY;
%         while 1
%             fg(sp)=abs(yy(1))-abs(yy(length(yy)));
%             if abs(fg(sp))<10
%                 break
%             else
%                 yy=YY-YY(sp+5);
%             end
%             if sp>ny-7
%                 spp=find(fg==min(fg));
%                 yy=YY-YY(spp+5);
%                 break
%             end
%             sp=sp+1;
%         end
%         xx=XX;
%         sp=1;
%         while 1
%             fg(sp)=abs(xx(1))-abs(xx(length(xx)));
%             if abs(fg(sp))<.01
%                 break
%             else
%                 xx=XX-XX(sp+5);
%             end
%             if sp>nx-7
%                 spp=find(fg==min(fg));
%                 xx=XX-XX(spp+5);
%                 break
%             end
%             sp=sp+1;
%         end
%         XX=xx;YY=yy;

        Z=[0;Z]/1000;
        nx=nx+1;
        ny=ny+1;
        nz=nz+1;
        AA(nx,:,:)=AA(nx-1,:,:);
        AA(:,ny,:)=AA(:,ny-1,:);
        AA(:,:,nz)=AA(:,:,nz-1);
    end
    aa=reshape(AA(:,:,lay),nx,ny);
    pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
    axis equal; axis tight
    title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    colormap(cmap); caxis(colim);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
    %update handles
    %     set(handles.axes1,'XLim',newXLim)
    %     set(handles.axes1,'YLim',newYLim)
    %
    guidata(hObject, handles)
end

% --- Executes on button press in center_stat.
function center_stat_Callback(hObject, eventdata, handles)
% hObject    handle to center_stat (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY x_orig y_orig AA lay

if length(XX)<1
    warndlg('You have to generate a model before you can center stations.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    %move stations to the center of the mesh
    if max(x)>0 && min(x)>0
        xc=max(x)-(max(x)-min(x))/2;
    elseif max(x)>0 && min(x)<0
        xc=max(x)-(max(x)+abs(min(x)))/2;
    else max(x)<0 && min(x)<0
        xc=(max(x)+min(x))/2;
    end
    if max(y)>0 && min(y)>0
        yc=max(y)-(max(y)-min(y))/2;
    elseif max(y)>0 && min(y)<0
        yc=max(y)-(max(y)+abs(min(y)))/2;
    else max(y)<0 && min(y)<0
        yc=(min(y)+max(y))/2;
    end

    if xc~=0 && yc ~=0
        x=x-xc;
        y=y-yc;
    else
        al_cen = questdlg('Already centered! Do you want to move stations manually?', ...
            'Move or Not', 'Yes', 'No','What do you mean?','');
        switch al_cen,
            case 'Yes',
                prompt={'distance in EW:','distance in NS'};
                name='In km';
                numlines=1;
                defaultanswer={'0','0'};
                answer=str2num(char(inputdlg(prompt,name,numlines,defaultanswer)));
                x=x+answer(1);
                y=y+answer(2);
            case 'No',
                disp('Your favorite color is Blue.')
            case 'What do you mean?',
                h = helpdlg('When making a new mesh for 3D inversion it is the best to have the stations centered in the mesh. In some cases you may want to shift all the stations by some distance towards any direction. Click yes to previous menu and enter these distances.','Move stations or not');
        end % switch
    end
    aa=reshape(AA(:,:,lay),nx,ny);
    pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
    title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    axis equal; axis tight
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
    x_orig=x; y_orig=y;
    %update handles
    guidata(hObject, handles)
end




% --- Executes on slider movement.
function mesh_rot_Callback(hObject, eventdata, handles)
% hObject    handle to mesh_rot (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider
global x y per nx ny nz Z XX YY res x_orig y_orig AA lay rot_cor_ang

if length(x)<1
    warndlg('You have to load stations before you can rotate anything!.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    rot_cor_ang=get(hObject,'Value');
    c = cos(rot_cor_ang*pi/180);    s = sin(rot_cor_ang*pi/180);
    rotmat=[c -s;s c];
    rotated=rotmat*[x_orig' y_orig']';
    x=rotated(1,:);y=rotated(2,:);

    newXLim=get(handles.axes1,'XLim');
    newYLim=get(handles.axes1,'YLim');
    if length(XX)>1
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on;  plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        caxis(colim); colormap(cmap);
        hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
        set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
            'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
        set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
        title(['rotation angle',num2str(rot_cor_ang),' degree']);
    else
        plot(x,y,'vk','markerfacecolor','b'); hold off
        title(['rotation angle',num2str(rot_cor_ang),' degree']);
    end

    axis equal
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)
    %update handles
    guidata(hObject, handles)
end



% --- Executes on button press in up_space.
function up_space_Callback(hObject, eventdata, handles)
% hObject    handle to up_space (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay use_it undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting upper-space with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        [up_x,up_y,click]=ginput(1);
        if click==1
            gh=(find(up_y<YY));
            AA(1:nx,gh(1):ny,lay)=10^res;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight;            title(['Layer= ',num2str(lay)]);
            caxis(colim); colormap(cmap);
            hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
            set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
            set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
        end
    end

    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in lower_space.
function lower_space_Callback(hObject, eventdata, handles)
% hObject    handle to lower_space (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['paint lower-space with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)]);
        [up_x,up_y,click]=ginput(1);
        if click==1
            gh=flipud(find(up_y>YY));
            AA(1:nx,1:gh(1),lay)=10^res;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight;             title(['Layer= ',num2str(lay)]);
            caxis(colim); colormap(cmap);
            hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
            set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
            set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
        end
    end

    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in left_space.
function left_space_Callback(hObject, eventdata, handles)
% hObject    handle to left_space (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting upper-space with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)]);
        [up_x,up_y,click]=ginput(1);

        %                 [up_x,up_y,click]=ginput(1);
        %         if click==1
        %             gh=(find(up_x>XX));
        %             AA(gh(1):nx,1:ny,lay)=10^res;


        if click==1
            gh=flipud(find(up_x<XX));
            AA(1:gh(1),1:ny,lay)=10^res;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight;             title(['Layer= ',num2str(lay)]);
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
            caxis(colim); colormap(cmap);
            hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
            set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
            set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
        end
    end
    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in right_space.
function right_space_Callback(hObject, eventdata, handles)
% hObject    handle to right_space (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting upper-space with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)]);
        [up_x,up_y,click]=ginput(1);
        if click==1
            gh=(find(up_x<XX));
            AA(gh(1):nx,1:ny,lay)=10^res;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight;             title(['Layer= ',num2str(lay)]);
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
            caxis(colim); colormap(cmap);
            hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
            set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
            set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
        end
    end
    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in column.
function column_Callback(hObject, eventdata, handles)
% hObject    handle to column (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting column with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_x<XX));
                AA(gh(1)-1,:,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                break
            end
        end
    end
    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end
% --- Executes on button press in row.
function row_Callback(hObject, eventdata, handles)
% hObject    handle to row (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting row with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_y<YY));
                AA(:,gh(1)-1,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                break
            end
        end
    end
    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end
% --- Executes on button press in upper_col.
function upper_col_Callback(hObject, eventdata, handles)
% hObject    handle to upper_col (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res1=ceil(res/10)*10;
        elseif res>100
            res1=ceil(res/100)*100;
        else
            res1=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting upper-column with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_x<XX)); ggh=find(up_y<YY);
                AA(gh(1)-1,ggh(1)-1:ny,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                break
            end
        end
    end
    zooms= zoom;; set(zooms,'Enable','on');
    %update handles.
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in lower_col.
function lower_col_Callback(hObject, eventdata, handles)
% hObject    handle to lower_col (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting lower-column with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_x<XX)); ggh=find(up_y<YY);
                AA(gh(1)-1,1:ggh(1)-1,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                break
            end
        end
    end
    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in left_row.
function left_row_Callback(hObject, eventdata, handles)
% hObject    handle to left_row (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting row with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_y<YY)); ggh=find(up_x<XX);
                AA(1:ggh(1)-1,gh(1)-1,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                zooms= zoom; set(zooms,'Enable','on');
                break
            end
        end
    end
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in right_row.
function right_row_Callback(hObject, eventdata, handles)
% hObject    handle to right_row (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting right-row with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_y<YY)); ggh=find(up_x<XX);
                AA(ggh(1)-1:nx,gh(1)-1,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                zooms= zoom;; set(zooms,'Enable','on');
                break
            end
        end
    end
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)

    guidata(hObject, handles)
end

% --- Executes on button press in copy_layer.
function copy_layer_Callback(hObject, eventdata, handles)
% hObject    handle to copy_layer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay lay_to_c undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    lay_to_c =str2num(get(handles.layers_past,'string')); % Get the resistivity
    if isempty(lay_to_c)==0
        if lay<lay_to_c && lay_to_c>0 && lay_to_c<nz+1
            for ius=lay:lay_to_c
                AA(:,:,ius)=AA(:,:,lay);
            end
            lay=lay_to_c;
            newXLim = get(handles.axes1,'XLim');
            newYLim = get(handles.axes1,'YLim');
            view_model_Callback(hObject, eventdata, handles)
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
        elseif lay>lay_to_c && lay_to_c>0 && lay_to_c<nz+1
            for ius=lay_to_c:lay
                AA(:,:,ius)=AA(:,:,lay);
            end
            lay=lay_to_c;
            newXLim = get(handles.axes1,'XLim');
            newYLim = get(handles.axes1,'YLim');
            view_model_Callback(hObject, eventdata, handles)
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
        elseif lay_to_c>nz | lay_to_c<1
            warndlg('Invalid layer number.')
        end
    else
        warndlg('Enter the layer number in "layer" field.')
    end
    zooms= zoom;; set(zooms,'Enable','on');

    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    guidata(hObject, handles)
end

% --- Executes on button press in layers_past.
function layers_past_Callback(hObject, eventdata, handles)
% hObject    handle to layers_past (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay
if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
end

% --- Executes on button press in jump_to_layer.
function jump_to_layer_Callback(hObject, eventdata, handles)
% hObject    handle to jump_to_layer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA undo

if length(XX)<5
    warndlg('Generate a model then view it.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    if min(XX)>0 %this means you have just loaded data from winglink
        cvx=cumsum(flipud(XX(1:nx/2)));
        XX=[flipud(-cvx); 0; cvx]/1000;
        cvy=cumsum(flipud(YY(1:ny/2)));
        YY=[flipud(-cvy); 0; cvy]/1000;
        Z=[0;Z]/1000;
        nx=nx+1;
        ny=ny+1;
        nz=nz+1;
        AA(nx,:,:)=AA(nx-1,:,:);
        AA(:,ny,:)=AA(:,ny-1,:);
        AA(:,:,nz)=AA(:,:,nz-1);
    end
    lay =str2double(get(handles.layers_past,'string')); % Get the resistivity
    if lay>0 & lay<nz+1
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    else
        warndlg('Invalid layer number.')
        lay=1;
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    end
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

    zooms= zoom;; set(zooms,'Enable','on');
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    guidata(hObject, handles)
end

% --- Executes on button press in previous_layer.
function previous_layer_Callback(hObject, eventdata, handles)
% hObject    handle to previous_layer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    if lay>1
        lay=lay-1;
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    else
        warndlg('Invalid layer number.')
        lay=1;
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    end
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

end
zooms= zoom;; set(zooms,'Enable','on');
%update handles
undo=undo+1;
savemodel_Call(hObject, eventdata, handles)

guidata(hObject, handles)

% --- Executes on button press in nex_layer.
function nex_layer_Callback(hObject, eventdata, handles)
% hObject    handle to nex_layer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    if lay<nz-1
        lay=lay+1;
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    else
        warndlg('Invalid layer number.')
        lay=1;
        aa=reshape(AA(:,:,lay),nx,ny);
        pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
        axis equal; axis tight
        title(['X=',num2str(nx-1),', Y=',num2str(ny-1),', Z=',num2str(nz-1),' layer=',num2str(lay)]);
    end
    caxis(colim); colormap(cmap);
    hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
    set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
        'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
    set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);

end
zooms= zoom;; set(zooms,'Enable','on');
%update handles
undo=undo+1;
savemodel_Call(hObject, eventdata, handles)
guidata(hObject, handles)

function resistivity_Callback(hObject, eventdata, handles)
% hObject    handle to resistivity (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of resistivity as text
%        str2double(get(hObject,'String')) returns contents of resistivity as a double
global x y per nx ny nz Z XX YY res AA lay


% --- Executes during object creation, after setting all properties.
function resistivity_CreateFcn(hObject, eventdata, handles)
% hObject    handle to resistivity (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in use_this_res.
function use_this_res_Callback(hObject, eventdata, handles)
% hObject    handle to use_this_res (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of use_this_res
global x y per nx ny nz Z XX YY res AA lay

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
end


% --- Executes on button press in cell_paint.
function cell_paint_Callback(hObject, eventdata, handles)
% hObject    handle to cell_paint (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo


if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    newXLim = get(handles.axes1,'XLim');
    newYLim = get(handles.axes1,'YLim');
    view_model_Callback(hObject, eventdata, handles)
    set(handles.axes1,'XLim',newXLim)
    set(handles.axes1,'YLim',newYLim)

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);
    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        newXLim=get(handles.axes1,'XLim');
        newYLim=get(handles.axes1,'YLim');
        title(['painting cell with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
        while 1
            [up_x,up_y,click]=ginput(1);
            if click==1
                gh=(find(up_y<YY)); ggh=find(up_x<XX);
                AA(ggh(1)-1,gh(1)-1,lay)=10^res;
                aa=reshape(AA(:,:,lay),nx,ny);
                pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
                axis equal; axis tight;                 title(['Layer= ',num2str(lay)]);
                set(handles.axes1,'XLim',newXLim)
                set(handles.axes1,'YLim',newYLim)
                caxis(colim); colormap(cmap);
                hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
                set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                    'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
                set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
            else
                zooms= zoom;; set(zooms,'Enable','on');
                break
            end
        end
    end
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    %update handles
    guidata(hObject, handles)
end

% --- Executes on button press in area.
function area_Callback(hObject, eventdata, handles)
% hObject    handle to area (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay undo

if length(XX)<5
    warndlg('Generate a model then modify it using these butttons.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    use_it= get(handles.use_this_res,'value');
    if use_it==1
        res =str2double(get(handles.resistivity,'string')); % Get the resistivity
        click=1;
    else
        while 1
            title('Pick color.');
            [bos res click]=ginput(1); res=10^res;
            if res<100000 & res>0.01
                break
            end
        end
        if res<100 & res>5
            res=ceil(res/10)*10;
        elseif res>100
            res=ceil(res/100)*100;
        else
            res=ceil(res*10)/10;
        end
        set(handles.resistivity,'string',num2str(res))
    end
    res=log10(res);

    if click==1
        if length(lay)<1
            lay=1; %First layer
        end
        while 1
            newXLim=get(handles.axes1,'XLim');
            newYLim=get(handles.axes1,'YLim');
            title(['area to paint with ',num2str(10^res),' ohm.m, Layer= ',num2str(lay)])
            %Area selection
            ku = waitforbuttonpress;
            point1 = get(gca,'CurrentPoint');    % button down detected
            finalRect = rbbox;                   % return figure units
            point2 = get(gca,'CurrentPoint');    % button up detected
            point1 = point1(1,1:2);              % extract x and y
            point2 = point2(1,1:2);
            p1 = min(point1,point2);             % calculate locations
            offset = abs(point1-point2);         % and dimensions
            xu = [p1(1) p1(1)+offset(1) p1(1)+offset(1) p1(1) p1(1)];
            yu = [p1(2) p1(2) p1(2)+offset(2) p1(2)+offset(2) p1(2)];
            %hold on; plot(xu,yu)                            % redraw in dataspace units
            if xu(1)<xu(3)
                yan=find(XX>xu(1) & XX<xu(3));sol=min(yan); sag=max(yan)-1;
            else
                yan=find(XX<xu(1) & XX>xu(3));sol=min(yan); sag=max(yan)-1;
            end

            if yu(1)<yu(3)
                yan=find(YY>yu(1) & YY<yu(3));alt=min(yan); ust=max(yan)-1;
            else
                yan=find(YY<yu(1) & YY>yu(3));alt=min(yan); ust=max(yan)-1;
            end
            if length(sol)<1 || length(sag)<1 || length(ust)<1 || length(alt)<1
                warndlg('You have finished area painting.')
                zooms= zoom;; set(zooms,'Enable','on');
                break
            end
            AA(sol:sag,alt:ust,lay)=10^res;
            aa=reshape(AA(:,:,lay),nx,ny);
            pcolor(XX,YY,log10(aa)'); hold on; plot(x,y,'vk','markerfacecolor','b'); hold off
            axis equal; axis tight;             title(['Layer= ',num2str(lay)]);
            set(handles.axes1,'XLim',newXLim)
            set(handles.axes1,'YLim',newYLim)
            caxis(colim); colormap(cmap);
            hcb=colorbar('ver','position',[0.7 0.28 0.02309 0.1977]);
            set(hcb,'YTick',[-1,-.5,0 0.5,1,1.5,2,2.5,3,3.5,4],...
                'YTickLabel',{'0.1','0.3','1','3','10','30','100','300','1000','3000','10000'});
            set(get(hcb, 'Title'), 'String', 'Resistivity (\Omegam)','fontsize',12);
        end
    end
    %update handles
    undo=undo+1;
    savemodel_Call(hObject, eventdata, handles)
    guidata(hObject, handles);
end

function max_per_Callback(hObject, eventdata, handles)
% hObject    handle to max_per (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of max_per as text
%        str2double(get(hObject,'String')) returns contents of max_per as a double


% --- Executes during object creation, after setting all properties.
function max_per_CreateFcn(hObject, eventdata, handles)
% hObject    handle to max_per (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function min_per_Callback(hObject, eventdata, handles)
% hObject    handle to min_per (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of min_per as text
%        str2double(get(hObject,'String')) returns contents of min_per as a double


% --- Executes during object creation, after setting all properties.
function min_per_CreateFcn(hObject, eventdata, handles)
% hObject    handle to min_per (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function error_floor_Callback(hObject, eventdata, handles)
% hObject    handle to error_floor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of error_floor as text
%        str2double(get(hObject,'String')) returns contents of error_floor as a double


% --- Executes during object creation, after setting all properties.
function error_floor_CreateFcn(hObject, eventdata, handles)
% hObject    handle to error_floor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in plot_it.
function plot_it_Callback(hObject, eventdata, handles)
% hObject    handle to plot_it (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global x y per nx ny nz Z XX YY res AA lay matfile rot_cor_ang

warning off

%starts definig the data as in save_data...
max_per =str2num(get(handles.max_per,'string'));
min_per =str2num(get(handles.min_per,'string'));
per_decade =str2num(get(handles.per_decade,'string'));
err_flr =.01*str2num(get(handles.error_floor,'string')); %diagonal
err_flr2=.01*str2num(get(handles.error_floor2,'string')); %off-diagonal (this is correct for sure)
f_t =(get(handles.full_tensor,'value')); %full tensor or not number of response is 8 if f_t=1, and4 if f_t=0

if length(x)<5
    warndlg('Generate a model then prepare your data.')
elseif length(max_per)<1 || length(min_per)<1 || length(per_decade)<1 || length(err_flr)<1
    warndlg('Enter data parameters first.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    font_n='arial'; %to open without warning in AI CS3
    font_s=5;
    lw=0.5; %linewidth
    ms=2;%markersize

    eval(['load ',matfile]);
    hold off; plot(x,y,'vk','markerfacecolor','b'); text(x,y,site,'fontsize',6);
    axis equal;axis([min(x)-sttol max(x)+sttol min(y)-sttol max(y)+sttol]);
    xlabel('Easting (km)'); ylabel('Northing (km)')
    stps=1;
    while 1
        title('select stations to plot');
        %Area selection
        ku = waitforbuttonpress;
        point1 = get(gca,'CurrentPoint');    % button down detected
        finalRect = rbbox;                   % return figure units
        point2 = get(gca,'CurrentPoint');    % button up detected
        point1 = point1(1,1:2);              % extract x and y
        point2 = point2(1,1:2);
        p1 = min(point1,point2);             % calculate locations
        offset = abs(point1-point2);         % and dimensions
        xu = [p1(1) p1(1)+offset(1) p1(1)+offset(1) p1(1) p1(1)];
        yu = [p1(2) p1(2) p1(2)+offset(2) p1(2)+offset(2) p1(2)];
        if xu(1)<min(x)-6 || xu(3)>max(x)+6 || yu(1)< min(y)-6 || yu(3)>max(y)+6
            %             if length(stps)>3
            %                 stps=stps(1:3);
            %                 warndlg('This version can only plot 6 sites at once.')
            %             end
            break
        end

        %plot(xu,yu)                            % redraw in dataspace units
        stp1=find(y>yu(1) & y<yu(3) & x>xu(1) & x<xu(2));
        if stp1>0
            dec=find(stps==stp1);
            if dec>0
                stps(dec)=[];
            else
                stps=[stps,stp1];
            end
            hold on; plot(x,y,'vk','markerfacecolor','b');
            plot(x(stps),y(stps),'vk','markerfacecolor','r');  text(x,y,site,'fontsize',6);
        end
    end

    xtop=x(stps); ytop=y(stps);
    if f_t==1
        NR=8;
    else
        NR=4;
    end
    eval(['load ',matfile]);
    
    %====================Rotate impedance data to opposite direction of the mesh===============================
    if rot_cor_ang~=[]
        c = cos(rot_cor_ang*pi/180.);
        s = sin(rot_cor_ang*pi/180.);
        U = [ c*c ,  c*s ,  c*s , s*s  ; -c*s ,  c*c , -s*s , c*s  ; -c*s , -s*s ,  c*c , c*s  ; s*s , -c*s , -c*s , c*c ];
        Uv = [ c  , s ; -s  , c ];
        for tpr=1:length(site)
            ZZZ(:,:,tpr)=U*squeeze(ZZZ(:,:,tpr));
            EEE(:,:,tpr)=(U.^2*squeeze(EEE(:,:,tpr)).^2).^.5;
            ZZZ_HZ(:,:,tpr)=Uv*squeeze(ZZZ_HZ(:,:,tpr));
            EEE_HZ(:,:,tpr)=(Uv.^2*squeeze(EEE_HZ(:,:,tpr)).^2).^.5;
        end
    end
    EEE=EEE(:,:,stps); EEE_HZ=EEE_HZ(:,:,stps); ZZZ=ZZZ(:,:,stps); ZZZ_HZ=ZZZ_HZ(:,:,stps); coord=coord(stps,:); site=site(:,stps);
    EEE=sqrt(EEE)/796; ZZZ=conj(ZZZ(:,:,:)/796); ZZZ_HZ=ZZZ_HZ(:,:,:); EEE_HZ=EEE_HZ(:,:,:);  %Tipper is not implemented
    % figure(1);subplot(221);loglog(T_n,abs(ZZZ_n(2,:,1)),'o');hold on;
    % figure(1);subplot(222);loglog(T_n,abs(ZZZ_n(2,:,3)),'o');hold on;

    minid=max(flipud(find(min_per>T)));
    maxid=min(find(max_per<T));
    if per_decade==2
        frtp=minid:4:maxid;
    elseif per_decade==3
        frtp=minid:3:maxid;
    elseif per_decade==4
        frtp=minid:2:maxid;
    else
        warndlg('You can only have 2, 3 or 4 periods per decade (will be updated soon).')
    end
    if exist('frtp')==1
        np=length(frtp);   %number of periods
        T=T(frtp);         %updating periods
        EEE=EEE(:,frtp,:); EEE_HZ=EEE_HZ(:,frtp,:); ZZZ=ZZZ(:,frtp,:); ZZZ_HZ=ZZZ_HZ(:,frtp,:);
    end
    nfim=length(T);
    for ic=1:4
        for itrpo=1:length(stps)
            for iu=1:nfim   %number of frequencies
                if ic==8 | ic==8    %ic is never 8 change to 1 and 4 if needed so program uses 'else' and after
                    if EEE(ic,iu,itrpo)<err_flr2*sqrt(abs(ZZZ(1,iu,itrpo).*ZZZ(4,iu,itrpo)))
                        EEE(ic,iu,itrpo)=err_flr2*sqrt(abs(ZZZ(1,iu,itrpo).*ZZZ(4,iu,itrpo)));   %10 errorfloor
                    end
                elseif ic==1 | ic==4
                    %four times more errorbar
                    %if EEE(ic,iu,itrpo)<errflr*sqrt((ZZZ(2,iu,itrpo).*conj(ZZZ(3,iu,itrpo))))
                    EEE(ic,iu,itrpo)=err_flr*sqrt(abs(ZZZ(ic,iu,itrpo).*ZZZ(ic,iu,itrpo)));   %10 errorfloor
                    %end
                else
                    EEE(ic,iu,itrpo)=err_flr2*sqrt(abs(ZZZ(ic,iu,itrpo).*ZZZ(ic,iu,itrpo)));   %10 errorfloor
                end
            end
        end

    end
    for stg=1:4
        for yt=1:length(stps)
            sjk=reshape(ZZZ(stg,:,:).*conj(ZZZ(stg,:,:)),np,length(stps));
            rho(:,yt,stg)=(1*T/2/pi/4/pi/10^-7.*sjk(:,yt));
            err(:,yt,stg)=rho(:,yt,stg)'.*EEE(stg,:,yt)./abs(ZZZ(stg,:,yt));
        end
    end
    %start plotting
    hold off;
    zv=[1 2 3 7 8 9];
    if length(stps)<11 & length(stps)>4
        figure(3)
        set(gca,'fontname',font_n,'fontsize',font_s); hold on;
        for zm=1:length(stps)
            subplot(3,3,zm) ;
            loglog(T',rho(:,zm,2),'vk','markersize',ms,'markerfacecolor','none'); hold on
            loglog(T',rho(:,zm,3),'ok','markersize',ms,'markerfacecolor','none');

            legend('XY','YX','orientation','vertical','fontname',font_n)
            legend boxoff
            errorbarloglog(T',rho(:,zm,2),err(:,zm,2),'none','v',ms,'k');
            errorbarloglog(T',rho(:,zm,3),err(:,zm,3),'none','o',ms,'k');
            if NR==8
                loglog(T,rho(:,zm,1),'vb','markersize',ms,'markerfacecolor','none');
                loglog(T,rho(:,zm,4),'ob','markersize',ms,'markerfacecolor','none');
            end
            axis([T(1)-T(1)/2 T(length(T))+T(length(T))/2 1 4000]); title(site(zm));
            xlabel('Period (sec.)'); ylabel('App. Res. (\Omegam)');
        end
    elseif length(stps)<4
        figure(2)
        set(gca,'fontname',font_n,'fontsize',font_s); hold on;
        for zm=1:length(stps)
            subplot(2,3,zv(zm));
            loglog(T',rho(:,zm,2),'vk','markersize',ms,'markerfacecolor','none');hold on;
            loglog(T',rho(:,zm,3),'ok','markersize',ms,'markerfacecolor','none');
            legend('XY','YX','orientation','vertical','fontname',font_n)
            legend boxoff

            loglog(T,rho(:,zm,1),'vb','markersize',ms,'markerfacecolor','none');
            loglog(T,rho(:,zm,4),'ob','markersize',ms,'markerfacecolor','none');

            errorbarloglog(T',rho(:,zm,2),err(:,zm,2),'none','v',ms,'k');
            errorbarloglog(T',rho(:,zm,3),err(:,zm,3),'none','o',ms,'k');

            axis([T(1)-T(1)/2 T(length(T))+T(length(T))/2 1 4000]); title(site(zm));
            xlabel('Period (sec.)'); ylabel('App. Res. (\Omegam)'); box on;

            global rho err T ZZZ
            subplot(4,3,zv(zm+3));
            poles=180/pi*(angle(ZZZ(2,:,zm)/796))+90;
            poles2=180/pi*(angle(ZZZ(3,:,zm)/796))-90;
            h2=errorbarloglog_phase(T',poles',err(:,zm,2)./rho(:,zm,2)*100*0.29,'none','v',ms,'k'); hold on;
            errorbarloglog_phase(T',poles2',err(:,zm,3)./rho(:,zm,3)*100*0.29,'none','o',ms,'k');
            axis([T(1)-T(1)/2 T(length(T))+T(length(T))/2 0 110]); title(site(zm));
            set(h2,'Yscale','linear');
            xlabel('Period (sec.)'); ylabel('Phase. (degree)'); box on
        end
    else
        warndlg('This version can only plot 9 sites at once.')
    end
end
%update handles
guidata(hObject, handles);


% --- Executes on button press in save_data.
function save_data_Callback(hObject, eventdata, handles)
% hObject    handle to save_data (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global x y per nx ny nz Z XX YY res AA lay matfile f_t rot_cor_ang

z_backup=Z;
if length(XX)<5
    warndlg('Generate a model then prepare your data.')
else
    [cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);

    max_per =str2num(get(handles.max_per,'string'));
    min_per =str2num(get(handles.min_per,'string'));
    per_decade =str2num(get(handles.per_decade,'string'));
    err_flr =.01*str2num(get(handles.error_floor,'string')); %diagonal
    err_flr2=.01*str2num(get(handles.error_floor2,'string')); %off-diagonal (this is correct for sure)
    if length(max_per)<1 || length(min_per)<1 || length(per_decade)<1 || length(err_flr)<1
        warndlg('Enter data parameters first.')
    else
        eval(['load ',matfile]);
        f_t =(get(handles.full_tensor,'value')); %full tensor or not number of response is 8 if f_t=1, and4 if f_t=0
        if f_t==1
            NR=8;
        else
            NR=4;
        end


        %====================Rotate impedance data to opposite direction of the mesh===============================
        
        if rot_cor_ang~=0
            c = cos(rot_cor_ang*pi/180.);
            s = sin(rot_cor_ang*pi/180.);
            U = [ c*c ,  c*s ,  c*s , s*s  ; -c*s ,  c*c , -s*s , c*s  ; -c*s , -s*s ,  c*c , c*s  ; s*s , -c*s , -c*s , c*c ];
            Uv = [ c  , s ; -s  , c ];
            for tpr=1:length(site)
                ZZZ(:,:,tpr)=U*squeeze(ZZZ(:,:,tpr));
                EEE(:,:,tpr)=(U.^2*squeeze(EEE(:,:,tpr)).^2).^.5;
                ZZZ_HZ(:,:,tpr)=Uv*squeeze(ZZZ_HZ(:,:,tpr));
                EEE_HZ(:,:,tpr)=(Uv.^2*squeeze(EEE_HZ(:,:,tpr)).^2).^.5;
            end
            warndlg(['MT impedances rotated ', num2str(rot_cor_ang),' degrees NE! Make sure when you export edis from S3DET_v5 you keep this azimuthal information in the edi files'])
            %winglink assumes mesh rotation is negative in NE and positive
            %in NW. Here program understands this convention but shows
            %always positive angles. If your winglink exported model has
            %-20 rotation, this program will correctly interpret the angle but will show you as positive 
        end

        EEE=sqrt(EEE)/796; ZZZ=conj(ZZZ(:,:,:)/796); ZZZ_HZ=ZZZ_HZ(:,:,:); EEE_HZ=EEE_HZ(:,:,:);  %Tipper is not implemented
        % figure(1);subplot(221);loglog(T_n,abs(ZZZ_n(2,:,1)),'o');hold on;
        % figure(1);subplot(222);loglog(T_n,abs(ZZZ_n(2,:,3)),'o');hold on;
        nfim=length(T);
        for ic=1:4
            for itrpo=1:length(x)
                for iu=1:nfim   %number of frequencies in *mat file
                    if ic==8 | ic==8    %ic is never 8 change to 1 and 4 if needed so program uses 'else' and after
                        if EEE(ic,iu,itrpo)<err_flr2*sqrt(abs(ZZZ(1,iu,itrpo).*ZZZ(4,iu,itrpo)))
                            EEE(ic,iu,itrpo)=err_flr2*sqrt(abs(ZZZ(1,iu,itrpo).*ZZZ(4,iu,itrpo)));   %10 errorfloor
                        end
                    elseif ic==1 | ic==4
                        %four times more errorbar
                        %if EEE(ic,iu,itrpo)<errflr*sqrt((ZZZ(2,iu,itrpo).*conj(ZZZ(3,iu,itrpo))))
                        EEE(ic,iu,itrpo)=err_flr*sqrt(abs(ZZZ(ic,iu,itrpo).*ZZZ(ic,iu,itrpo)));   %10 errorfloor
                        %end
                    else
                        EEE(ic,iu,itrpo)=err_flr2*sqrt(abs(ZZZ(ic,iu,itrpo).*ZZZ(ic,iu,itrpo)));   %10 errorfloor
                    end
                end
            end
        end
        minid=max(flipud(find(min_per>T)));
        maxid=min(find(max_per<T));
        if per_decade==2
            frtp=minid:4:maxid;
        elseif per_decade==3
            frtp=minid:3:maxid;
        elseif per_decade==4
            frtp=minid:2:maxid;
        else
            warndlg('You can only have 2, 3 or 4 periods per decade (will be updated soon).')
        end
        if exist('frtp')==1
            np=length(frtp);   %number of periods
            T=T(frtp);         %updating periods
            % T/2/pi/4/pi/10^-7.*(ZZZ(2,frtp,1).*conj(ZZZ(2,frtp,1)))'
            % figure(2);errorbar(log10(T),T/2/pi/4/pi/10^-7.*(ZZZ(2,frtp,1).*conj(ZZZ(2,frtp,1)))',T/2/pi/4/pi/10^-7.*(EEE(2,frtp,1).*conj(EEE(2,frtp,1)))')
            % set(gca, 'YScale', 'log');
            % figure(1);subplot(221);loglog(T,abs(ZZZ_n(2,1:26,1)),'.');hold on;
            % figure(1);subplot(222);loglog(T,abs(ZZZ_n(2,1:26,3)),'.');hold on;
            %---------------READ IMPEDENCE-----------------------------------------
            Zxx=ZZZ(1,:,:); RZxx=squeeze(real(Zxx)); IZxx=squeeze(imag(Zxx));
            CIZxx=[IZxx(frtp,:)'];
            CRZxx=[RZxx(frtp,:)'];

            EZxx=squeeze(EEE(1,:,:));
            EZxx1=([EZxx(frtp,:)']);
            EZxx2=([EZxx(frtp,:)']);
            %-------------------------------------------------------------------------
            Zxy=ZZZ(2,:,:); RZxy=squeeze(real(Zxy)); IZxy=squeeze(imag(Zxy));
            CIZxy=[IZxy(frtp,:)'];
            CRZxy=[RZxy(frtp,:)'];

            EZxy=squeeze(EEE(2,:,:));
            EZxy1=([EZxy(frtp,:)']);
            EZxy2=([EZxy(frtp,:)']);
            %-------------------------------------------------------------------------
            Zyx=ZZZ(3,:,:);RZyx=squeeze(real(Zyx));IZyx=squeeze(imag(Zyx));
            CIZyx=[IZyx(frtp,:)'];
            CRZyx=[RZyx(frtp,:)'];

            EZyx=squeeze(EEE(3,:,:));
            EZyx1=([EZyx(frtp,:)']);
            EZyx2=([EZyx(frtp,:)']);
            %-------------------------------------------------------------------------
            Zyy=ZZZ(4,:,:);RZyy=squeeze(real(Zyy));IZyy=squeeze(imag(Zyy));
            CIZyy=[IZyy(frtp,:)'];
            CRZyy=[RZyy(frtp,:)'];

            EZyy=squeeze(EEE(4,:,:));
            EZyy1=([EZyy(frtp,:)']);
            EZyy2=([EZyy(frtp,:)']);
            %------------write data---------------------------------------
            def = {'data_name'};
            prompt = {'enter data file name must be same as model name'};
            titles  = 'Data file name';
            datf = char(inputdlg(prompt,titles,1,def)); %model file name
            datf=[datf,'.data'];
            while exist(datf)>0
                newdata_name = questdlg('This file exist! Do you want to overwrite the file?', ...
                    'The data file exist', ...
                    'Yes', 'No','dummy');
                switch newdata_name,
                    case 'Yes',
                        eval(['delete ',datf]);
                        break
                    case 'No',
                        def = {'data_name'};
                        prompt = {'enter a different data file name (must be same as model name)'};
                        titles  = 'Data file name';
                        datf = char(inputdlg(prompt,titles,1,def)); %model file name
                        datf=[datf,'.data']
                end % switch
            end

            fid=fopen(datf,'w');
            fprintf(fid,'%d %d %d\n',length(x),length(frtp),NR);
            fprintf(fid,'%s\n','Station_Location: N-S');
            for kf=1:length(x)
                %kii=stind(kf);
                fprintf(fid,'%.2f %s',y(kf)*1000,' ');
                %if kii==7 | kii==14 | kii==21 | kii==28 | kii==35 | kii== 42 | kii==49
                %    fprintf(fid,'\n');
                %end
            end
            fprintf(fid,'\n');
            fprintf(fid,'%s\n','Station_Location: E-W');
            for kf=1:length(x)
                %kii=stind(kf);
                fprintf(fid,'%.2f %s',x(kf)*1000,' ');
                %if kii==7 | kii==14 | kii==21 | kii==28 | kii==35 | kii== 42 | kii==49
                %    fprintf(fid,'\n');
                %end
            end
            fprintf(fid,'\n');

            for I=1:length(frtp)
                text1='DATA_Period:       ';
                fprintf(fid,'%s %5d\n',text1,T(I));
                for sI=1:length(x)  %station number
                    if NR==8;
                        Z=[CRZxx(sI,I),CIZxx(sI,I),CRZxy(sI,I),CIZxy(sI,I),CRZyx(sI,I),CIZyx(sI,I),CRZyy(sI,I),CIZyy(sI,I)];
                        indNaNZ=find(isnan(Z)==1); Z(indNaNZ)=4.6326e-005;
                        zerZ=find(Z==0); Z(zerZ)=-1.8452e-005;
                        fprintf(fid,'%1.4d %1.4d %1.4d %1.4d %1.4d %1.4d %1.4d %1.4d\n',Z);  %Z is conjugated
                    else
                        Z=[CRZxy(sI,I),CIZxy(sI,I),CRZyx(sI,I),CIZyx(sI,I)];
                        indNaNZ=find(isnan(Z)==1); Z(indNaNZ)=4.6326e-005;
                        zerZ=find(Z==0); Z(zerZ)=-1.8452e-005;
                        fprintf(fid,'%1.4d %1.4d %1.4d %1.4d\n',Z);  %Z is conjugate
                    end
                end
            end
            for I=1:length(frtp)
                text1='ERROR_Period:       ';
                fprintf(fid,'%s %5d\n',text1,T(I));
                for sI=1:length(x)  %station number
                    if NR==8
                        EZ=[EZxx1(sI,I),EZxx2(sI,I),EZxy1(sI,I),EZxy2(sI,I),EZyx1(sI,I),EZyx2(sI,I),EZyy1(sI,I),EZyy2(sI,I)];
                        indNaNE=find(isnan(EZ)==1); EZ(indNaNE)=0.1;
                        fprintf(fid,'%d %d %d %d %d %d %d %d\n',EZ);
                    else
                        EZ=[EZxy1(sI,I),EZxy2(sI,I),EZyx1(sI,I),EZyx2(sI,I)];
                        indNaNE=find(isnan(EZ)==1); EZ(indNaNE)=0.1;
                        fprintf(fid,'%d %d %d %d\n',EZ);
                    end
                end
            end
            for I=1:length(frtp)
                text1='ERMAP_Period:       ';
                fprintf(fid,'%s %5d\n',text1,T(I));
                for sI=1:length(x)
                    if NR==8
                        ERc=[1,1,1,1,1,1,1,1];
                        fprintf(fid,'%d %d %d %d %d %d %d %d\n',ERc);
                    else
                        ERc=[1,1,1,1];
                        fprintf(fid,'%d %d %d %d\n',ERc);
                    end
                end
            end
            fclose(fid);
            N=length(x)*length(frtp)*NR;
            M=nx*ny*nz;
            MEM=ceil(1.2*(8*N^2+8*N*M)*10^-9*100);
            warndlg(['you need ',num2str(MEM/100),'(+- %10) GB of memeory for this model'])
        end
    end
    %update handles
    guidata(hObject, handles);
end
Z=z_backup;


% --- Executes during object deletion, before destroying properties.
function error_floor2_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to error_floor2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% --- Executes during object creation, after setting all properties.

function error_floor2_CreateFcn(hObject, eventdata, handles)
% hObject    handle to error_floor2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes during object deletion, before destroying properties.
function full_tensor_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to full_tensor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function full_tensor_CreateFcn(hObject, eventdata, handles)
% hObject    handle to full_tensor (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function save_data_CreateFcn(hObject, eventdata, handles)
% hObject    handle to save_data (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object deletion, before destroying properties.
function save_data_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to save_data (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object deletion, before destroying properties.
function min_per_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to min_per (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function cell_paint_CreateFcn(hObject, eventdata, handles)
% hObject    handle to cell_paint (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object deletion, before destroying properties.
function cell_paint_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to cell_paint (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object deletion, before destroying properties.
function text18_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to text18 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function text18_CreateFcn(hObject, eventdata, handles)
% hObject    handle to text18 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function text19_CreateFcn(hObject, eventdata, handles)
% hObject    handle to text19 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function text20_CreateFcn(hObject, eventdata, handles)
% hObject    handle to text20 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object deletion, before destroying properties.
function text20_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to text20 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function rotate_stations_text_CreateFcn(hObject, eventdata, handles)
% hObject    handle to rotate_stations_text (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object deletion, before destroying properties.
function rotate_stations_text_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to rotate_stations_text (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function data_out_CreateFcn(hObject, eventdata, handles)
% hObject    handle to data_out (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object deletion, before destroying properties.
function data_out_DeleteFcn(hObject, eventdata, handles)
% hObject    handle to data_out (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function layers_past_CreateFcn(hObject, eventdata, handles)
% hObject    handle to layers_past (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end
% --- Executes during object creation, after setting all properties.
function copy_layer_CreateFcn(hObject, eventdata, handles)
% hObject    handle to copy_layer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function area_CreateFcn(hObject, eventdata, handles)
% hObject    handle to area (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function addlayer_CreateFcn(hObject, eventdata, handles)
% hObject    handle to addlayer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function deletelayer_CreateFcn(hObject, eventdata, handles)
% hObject    handle to deletelayer (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function generatemodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to generatemodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function newmodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to newmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

function error_floor2_Callback(hObject, eventdata, handles)
% hObject    handle to error_floor2 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of error_floor2 as text
%        str2double(get(hObject,'String')) returns contents of error_floor2 as a double

% --- Executes on button press in def_p.
function def_p_Callback(hObject, eventdata, handles)
% hObject    handle to def_p (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of def_p

% --- Executes during object creation, after setting all properties.
function axes1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to axes1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: place code in OpeningFcn to populate axes1

% --- Executes during object creation, after setting all properties.
function loadmodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to loadmodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function figure1_CreateFcn(hObject, eventdata, handles)
% hObject    handle to figure1 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes during object creation, after setting all properties.
function num_laye_CreateFcn(hObject, eventdata, handles)
% hObject    handle to num_laye (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

function max_res_Callback(hObject, eventdata, handles)
% hObject    handle to max_res (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of max_res as text
%        str2double(get(hObject,'String')) returns contents of max_res as a double

% --- Executes during object creation, after setting all properties.
function max_res_CreateFcn(hObject, eventdata, handles)
% hObject    handle to max_res (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function min_res_Callback(hObject, eventdata, handles)
% hObject    handle to min_res (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of min_res as text
%        str2double(get(hObject,'String')) returns contents of min_res as a double

% --- Executes during object creation, after setting all properties.
function min_res_CreateFcn(hObject, eventdata, handles)
% hObject    handle to min_res (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function stat_tol_Callback(hObject, eventdata, handles)
% hObject    handle to stat_tol (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of stat_tol as text
%        str2double(get(hObject,'String')) returns contents of stat_tol as a double

% --- Executes during object creation, after setting all properties.
function stat_tol_CreateFcn(hObject, eventdata, handles)
% hObject    handle to stat_tol (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

% --- Executes on button press in con_tinue.
function con_tinue_Callback(hObject, eventdata, handles)
% hObject    handle to con_tinue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
uiresume(gcbf);
set(hObject,'visible','off')
guidata(hObject,handles)

% --- Executes during object creation, after setting all properties.
function con_tinue_CreateFcn(hObject, eventdata, handles)
% hObject    handle to con_tinue (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% --- Executes on button press in load_wsinv3dmt.
function load_wsinv3dmt_Callback(hObject, eventdata, handles)
% hObject    handle to load_wsinv3dmt (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This will load the previously saved models by this program for wsinv3dmt
clear x y per nx ny nz Z XX YY x_orig y_orig AA lay matfile mes_rot
global x y per nx ny nz Z XX YY x_orig y_orig AA lay matfile mes_rot undo undodan
lay=1;
[cmap colim sttol]=read_col_sttol(hObject, eventdata, handles);
set(handles.axes1,'HandleVisibility','ON');
axes(handles.axes1);
%Load model
if undodan>0 & undo>0
    meshf=['undo',num2str(undo)];
    clear undodan
else
    delete undo*
    [meshf,matpath] = uigetfile({'*'},'pick wsinv3dmt file');
end
fid=fopen(meshf,'r');
anlamsiz=fgetl(fid); %header # INITIAL MODEL
NN=str2num(fgetl(fid)); ny=round(NN(1)); nx=round(NN(2)); nz=round(NN(3));
%winglink uses mathematical coordinates WSINV3DMT uses geographical
%coordinates
YY=round(str2num(fgetl(fid)))'; XX=round(str2num(fgetl(fid)))'; Z=round(str2num(fgetl(fid)))';
res=round(str2num(fgetl(fid))*10)/10;
slj=[];sli=[]; indexler(1)=1;
while 1
    %fgetl(fid) %MATLAB related issue this line is needed sometimes
    indexler=str2num(fgetl(fid));
    sli=fscanf(fid,'%f',nx*ny);
    for klo=indexler(1):indexler(2)
        slj=[slj;sli];
    end
    anlamsiz=fgetl(fid);
    if indexler(2)==nz
        break
    end
    %anlamsiz=fgetl(fid)
end
fclose(fid);
AA=reshape(slj,nx,ny,nz);
AAA(:,1,:)=AA(:,ny,:);
AAA(:,ny,:)=AA(:,1,:);
for ty=2:ny-1
    AAA(:,ty,:)=AA(:,ny-ty+1,:);
end
AA=AAA;
%Program will only work for nine distinct resistivity values in model file
nnr= length(res);
for uy=1:nnr
    AA(find(AA==uy))=res(uy);
end
res=unique(slj);
if length(res)>9
    warndlg('You cannot have more than 9 resistivities-3D inversion won"t work','!! Warning !!');
end
% load Data
if undo>0
    view_model_Callback(hObject, eventdata, handles)
else
    delete undo*
    [matfile,matpath] = uigetfile({'*.mat'},'pick mat file');
    eval(['load ',matfile]);
    per = T; logper = log10(per); nper = length(per); nsites = size(coord,1);
    lat  = coord(:,2)'; long = coord(:,1)';
    cent_lat  = (max(lat)+min(lat))/2;
    cent_long = (max(long)+min(long))/2;
    % Convert degrees to km N-S and E-W
    x = 111.*cos(cent_lat*pi/180)*(long-cent_long);
    y = 111.*(lat-cent_lat);
    x_orig=x; y_orig=y;
    plot(x,y,'vk','markerfacecolor','b');
    axis([min(x)-sttol max(x)+sttol min(y)-sttol max(y)+sttol]);
    view_model_Callback(hObject, eventdata, handles)
    text(x,y,site,'fontsize',6);
end
%undo=undo;
savemodel_Call(hObject, eventdata, handles)

data=guidata(hObject);
guidata(hObject, handles);

% --- Executes on button press in pushbutton71.
function pushbutton71_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton71 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
%restarts the program
clear all;close all;M3DET_v7;
delete undo*

% --- Executes during object creation, after setting all properties.
function per_decade_CreateFcn(hObject, eventdata, handles)
% hObject    handle to per_decade (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

%TEMPORARY SAVING FOR UNDO AND REDO

% --- Executes on button press in undo.
function undo_Callback(hObject, eventdata, handles)
% hObject    handle to undo (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global undo undodan
%warndlg('you must be joking... start over :)')
%for iy=1:200

if undo>0
    undo=undo-1;
    if exist(['undo',num2str(undo)])
        undodan=1;
        load_wsinv3dmt_Callback(hObject, eventdata, handles)
        undodan=0;
    end
else
    warndlg('That is all!')
end
%savemodel_Callback(hObject, eventdata, handles,undo)

% --- Executes on button press in redo_button.
function redo_button_Callback(hObject, eventdata, handles)
% hObject    handle to redo_button (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
global undo undodan
undo=undo+1;
if exist(['undo',num2str(undo)])
    undodan=1;
    load_wsinv3dmt_Callback(hObject, eventdata, handles)
    undodan=0;
else
    undo=undo-1;
end

% --- Executes on button press in savemodel.
function savemodel_Call(hObject, eventdata, handles)
% hObject    handle to savemodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

global x y per nx ny nz Z XX YY AA res undo
%If you haven't make the model you can't save it
sto=0; sto1=0;
if min(XX)<0  %This will work if you generated the model instead of loading from winglink
    AAA=AA;xx=diff(XX)*1000;yy=diff(YY)*1000;AAA(nx,:,:)=[];AAA(:,ny,:)=[];
    AAA(:,:,nz)=[];nxx=nx-1;nyy=ny-1;nzz=nz-1;ZZ=Z*1000;ZZ(1)=[];
else %This will work if you loaded a winglink model
    AAA=AA;xx=XX;yy=YY;nxx=nx;nyy=ny;nzz=nz;ZZ=Z;
end
if length(XX)<5
    warndlg('Make a model before you save it!')
    return
else
    set(handles.axes1,'HandleVisibility','ON');
    axes(handles.axes1)
    %-------Model Verification needed for backuping too so not everything will be backed up------
    if rem(nxx,2)==1 & rem(nyy,2)==1
        %   warndlg('model has odd number of columns and rows, model not saved. make them even')
        sto1=1;
    elseif rem(nxx,2)==1
        %    warndlg('model has odd number of columns, model not saved. make it even')
        sto1=1;
    elseif rem(nyy,2)==1
        %     warndlg('model has odd number of rows, model not saved. make it even')
        sto1=1;
    end

    for i=1:nxx-1
        stv=find(xx(i)<x & x<xx(i+1));%  stations in the first vertical mesh cell
        if isempty(stv)==0
            for j=1:ny-1
                sty=find(yy(j)<y(stv) & y(stv)<yy(j+1)); % stations fall in the same cell
                if length(sty)>1
                    hold on;plot(x(stv(sty)),y(stv(sty)),'vr','markerfacecolor','r','markersize',12);hold off;
                    sto=1;
                end
            end
        end
    end
    ress=unique(AAA);
    %----------------saves initial model file---------------
    if sto==0 && sto1==0
        nnr=length(ress);
        for opi=1:length(ress)
            AAA(find(AAA==ress(opi)))=opi;
        end
        modf = ['undo',num2str(undo)]; %model file name
        aa=1;slj=reshape(AAA,nxx*nyy*nzz,1);
        for uu=1:nzz-1
            if slj(aa:uu*nxx*nyy)==slj(uu*nxx*nyy+1:(uu+1)*nxx*nyy)
            else
                dint(uu)=uu+1;
            end
            aa=uu*nxx*nyy+1;
        end
        if exist('dint')==1
            dint=dint(find(dint~=0));dint=[1 dint nzz+1];
        else
            dint=[1 5 nzz+1];nnr=2;ress=[ress ress];
        end
        AAA=reshape(slj,nxx,nyy,nzz);
        fid=fopen(modf,'w');
        fprintf(fid,'%s\n','# INITIAL MODEL');
        %this is strange here now--- X and Y will change here to match the
        %program requirements. This is very strange but IS correct.
        fprintf(fid,'%d %d %d %d\n',nyy,nxx,nzz,nnr);
        fprintf(fid,'%.0f ',yy);
        fprintf(fid,'\n');
        fprintf(fid,'%.0f ',xx);
        fprintf(fid,'\n');
        fprintf(fid,'%.0f ',ZZ);
        fprintf(fid,'\n');
        fprintf(fid,'%.0f ',ress);
        fprintf(fid,' ');
        fprintf(fid,'\n');
        %For layered model with up to 9 distinct resistivities
        for kj=1:length(dint)-1
            fprintf(fid,'%d %d\n',dint(kj),dint(kj+1)-1);    %layers
            for i=nyy:-1:1
                fprintf(fid,'%d ',AAA(:,i,dint(kj)));
                if kj==length(dint)-1 && i==1
                else
                    fprintf(fid,'\n');
                end
            end
        end
        fclose(fid);

        data=guidata(hObject);
        guidata(hObject, handles);
    end
end

% --- Executes during object creation, after setting all properties.
function savemodel_CreateFcn(hObject, eventdata, handles)
% hObject    handle to savemodel (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called




% --- Executes on button press in pushbutton73.
function pushbutton73_Callback(hObject, eventdata, handles)
% hObject    handle to pushbutton73 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% This function will write the startup file.
[data_file,matpath] = uigetfile({'*'},' choose data file');
[model_file,matpath] = uigetfile({'*'},' choose initial model file');

s_prompt = {'data file:','initial model file:','output file:','priori model file(leave empty for default):','control model index(leave empty for default):','Target R.M.S.:','Max number of iterations:'...
    ,'Model length scale(leave empty for default):','Lagrange info(leave empty for default):','Error tol. level(leave empty for default):'};
s_titles  = 'Choose startup parameters';
out_file=['out_',model_file];def='default';
s_def= {data_file,model_file,out_file,'','','1.','10','','',''}; %Turkiye
def_lim = char(inputdlg(s_prompt,s_titles,1,s_def));
for spa=4:10
    if def_lim(spa,1:2)=='  '
        def_lim(spa,1:7)='default'
        spa
    end
end
line1=['DATA_FILE           ',def_lim(1,:)];
line2=['OUTPUT_FILE         ',def_lim(3,:)];
line3=['INITIAL_MODEL_FILE  ',def_lim(2,:)];
line4=['PRIOR_MODEL_FILE    ',def_lim(4,:)];
line5=['CONTROL_MODEL_INDEX ',def_lim(5,:)];
line6=['TARGET_RMS          ',def_lim(6,:)];
line7=['MAX_NO_ITERATION    ',def_lim(7,:)];
line8=['MODEL_LENGTH_SCALE  ',def_lim(8,:)];
line9=['LAGRANGE_INFO       ',def_lim(9,:)];
line10=['ERROR_TOL_LEVEL     ',def_lim(10,:)];
lines=[line1;line2;line3;line4;line5;line6;line7;line8;line9;line10];
fid=fopen('startup','w')
for poi=1:10
    fprintf(fid,'%s\n',lines(poi,:));
    if poi==10
        fprintf(fid,'%s',lines(poi,:));
    end
end
fclose(fid);



function edit29_Callback(hObject, eventdata, handles)
% hObject    handle to edit29 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit29 as text
%        str2double(get(hObject,'String')) returns contents of edit29 as a double


% --- Executes during object creation, after setting all properties.
function edit29_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit29 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit30_Callback(hObject, eventdata, handles)
% hObject    handle to edit30 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit30 as text
%        str2double(get(hObject,'String')) returns contents of edit30 as a double


% --- Executes during object creation, after setting all properties.
function edit30_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit30 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit31_Callback(hObject, eventdata, handles)
% hObject    handle to edit31 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit31 as text
%        str2double(get(hObject,'String')) returns contents of edit31 as a double


% --- Executes during object creation, after setting all properties.
function edit31_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit31 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit32_Callback(hObject, eventdata, handles)
% hObject    handle to edit32 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit32 as text
%        str2double(get(hObject,'String')) returns contents of edit32 as a double


% --- Executes during object creation, after setting all properties.
function edit32_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit32 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit33_Callback(hObject, eventdata, handles)
% hObject    handle to edit33 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit33 as text
%        str2double(get(hObject,'String')) returns contents of edit33 as a double


% --- Executes during object creation, after setting all properties.
function edit33_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit33 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end



function edit34_Callback(hObject, eventdata, handles)
% hObject    handle to edit34 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit34 as text
%        str2double(get(hObject,'String')) returns contents of edit34 as a double


% --- Executes during object creation, after setting all properties.
function edit34_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit34 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in scale_auto.
function scale_auto_Callback(hObject, eventdata, handles)
% hObject    handle to scale_auto (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of scale_auto





function per_decade_Callback(hObject, eventdata, handles)
% hObject    handle to per_decade (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of per_decade as text
%        str2double(get(hObject,'String')) returns contents of per_decade as a double




% --- Executes on button press in checkbox5.
function checkbox5_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox5 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox5





function edit35_Callback(hObject, eventdata, handles)
% hObject    handle to edit35 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of edit35 as text
%        str2double(get(hObject,'String')) returns contents of edit35 as a double


% --- Executes during object creation, after setting all properties.
function edit35_CreateFcn(hObject, eventdata, handles)
% hObject    handle to edit35 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on button press in checkbox6.
function checkbox6_Callback(hObject, eventdata, handles)
% hObject    handle to checkbox6 (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of checkbox6


% --- Executes during object creation, after setting all properties.
function view_model_CreateFcn(hObject, eventdata, handles)
% hObject    handle to view_model (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called


